@@ -347,7 +347,6 @@ cached_dir_update_entry (CachedDir  *dir,
         {
           if (!desktop_entry_reload (tmp->data))
 	    {
-	      desktop_entry_unref (tmp->data);
 	      dir->entries = g_slist_delete_link (dir->entries, tmp);
 	    }
 
@@ -1288,107 +1287,6 @@ entry_directory_list_get_all_desktops (EntryDirectoryList *list,
   entry_directory_list_add (list, get_all_func, set, NULL);
 }
 
-static gboolean
-get_by_category_func (EntryDirectory  *ed,
-                      DesktopEntry    *entry,
-                      const char      *relative_path,
-                      const char      *file_id,
-                      DesktopEntrySet *set,
-                      gpointer         user_data)
-{
-  const char *category = user_data;
-
-  if (ed->is_legacy && !desktop_entry_has_categories (entry))
-    {
-      if (strcmp (category, "Legacy") != 0)
-        return TRUE;
-
-      entry = desktop_entry_copy (entry);
-      desktop_entry_add_legacy_category (entry);
-    }
-  else
-    {
-      if (!desktop_entry_has_category (entry, category))
-        return TRUE;
-
-      entry = desktop_entry_ref (entry);
-    }
-
-  desktop_entry_set_add_entry (set,
-                               entry,
-                               file_id ? file_id : relative_path);
-  desktop_entry_unref (entry);
-
-  return TRUE;
-}
-
-void
-entry_directory_list_get_by_category (EntryDirectoryList *list,
-                                      const char         *category,
-                                      DesktopEntrySet    *set)
-{
-  menu_verbose (" Storing list %p category %s in set %p\n",
-                list, category, set);
-
-  entry_directory_list_add (list,
-                            get_by_category_func,
-                            set,
-                            (char *) category);
-}
-
-static gboolean
-get_inverse_func (EntryDirectory  *ed,
-                  DesktopEntry    *entry,
-                  const char      *relative_path,
-                  const char      *file_id,
-                  DesktopEntrySet *set,
-                  gpointer         user_data)
-{
-  DesktopEntrySet *inverse = user_data;
-
-  /* if not in the original set, add to inverse set */
-
-  if (desktop_entry_set_lookup (set, file_id ? file_id : relative_path) != NULL)
-    return TRUE;
-
-  if (ed->is_legacy && !desktop_entry_has_categories (entry))
-    {
-      entry = desktop_entry_copy (entry);
-      desktop_entry_add_legacy_category (entry);
-    }
-  else
-    {
-      entry = desktop_entry_ref (entry);
-    }
-
-  desktop_entry_set_add_entry (inverse,
-                               entry,
-                               file_id ? file_id : relative_path);
-  desktop_entry_unref (entry);
-
-  return TRUE;
-}
-
-void
-entry_directory_list_invert_set (EntryDirectoryList *list,
-                                 DesktopEntrySet    *set)
-{
-  DesktopEntrySet *inverse;
-
-  menu_verbose (" Inverting set %p relative to list %p\n",
-                set, list);
-
-  inverse = desktop_entry_set_new ();
-  entry_directory_list_add (list,
-                            get_inverse_func,
-                            set,
-                            inverse);
-
-  desktop_entry_set_swap_contents (set, inverse);
-
-  desktop_entry_set_unref (inverse);
-}
-
 void
 entry_directory_list_add_monitors (EntryDirectoryList        *list,
                                    EntryDirectoryChangedFunc  callback,
