@@ -102,6 +102,9 @@ static void      menu_tree_resolve_files        (MenuTree       *tree,
 						 MenuLayoutNode *layout);
 static void      menu_tree_force_recanonicalize (MenuTree       *tree);
 static void      menu_tree_invoke_monitors      (MenuTree       *tree);
+
+static void menu_tree_entry_unref_and_unset_parent     (MenuTreeEntry     *entry);
+static void menu_tree_directory_unref_and_unset_parent (MenuTreeDirectory *directory);
      
 /*
  * The idea is that we cache the menu tree for either a given
@@ -963,13 +966,13 @@ menu_tree_directory_unref (MenuTreeDirectory *directory)
   if (--directory->refcount == 0)
     {
       g_slist_foreach (directory->subdirs,
-                       (GFunc) menu_tree_directory_unref,
+                       (GFunc) menu_tree_directory_unref_and_unset_parent,
                        NULL);
       g_slist_free (directory->subdirs);
       directory->subdirs = NULL;
 
       g_slist_foreach (directory->entries,
-                       (GFunc) menu_tree_entry_unref,
+                       (GFunc) menu_tree_entry_unref_and_unset_parent,
                        NULL);
       g_slist_free (directory->entries);
       directory->entries = NULL;
@@ -987,6 +990,16 @@ menu_tree_directory_unref (MenuTreeDirectory *directory)
     }
 }
 
+static void
+menu_tree_directory_unref_and_unset_parent (MenuTreeDirectory *directory)
+{
+  g_return_if_fail (directory != NULL);
+
+  directory->parent = NULL;
+
+  menu_tree_directory_unref (directory);
+}
+
 static int
 menu_tree_directory_compare (MenuTreeDirectory *a,
                              MenuTreeDirectory *b)
@@ -1016,7 +1029,7 @@ menu_tree_entry_new (MenuTreeDirectory *parent,
 
   retval = g_new0 (MenuTreeEntry, 1);
 
-  retval->parent          = menu_tree_directory_ref (parent);
+  retval->parent          = parent;
   retval->desktop_entry   = desktop_entry_ref (desktop_entry);
   retval->desktop_file_id = g_strdup (desktop_file_id);
   retval->refcount        = 1;
@@ -1050,14 +1063,22 @@ menu_tree_entry_unref (MenuTreeEntry *entry)
         desktop_entry_unref (entry->desktop_entry);
       entry->desktop_entry = NULL;
 
-      if (entry->parent)
-	menu_tree_directory_unref (entry->parent);
       entry->parent = NULL;
 
       g_free (entry);
     }
 }
 
+static void
+menu_tree_entry_unref_and_unset_parent (MenuTreeEntry *entry)
+{
+  g_return_if_fail (entry != NULL);
+
+  entry->parent = NULL;
+
+  menu_tree_entry_unref (entry);
+}
+
 static int
 menu_tree_entry_compare (MenuTreeEntry *a,
                          MenuTreeEntry *b)
@@ -1997,6 +2018,8 @@ menu_tree_strip_duplicate_children (MenuTree       *tree,
   tmp = simple_nodes;
   while (tmp != NULL)
     {
+      GSList *next = tmp->next;
+
       if (prev)
         {
           MenuLayoutNode *p = prev->data;
@@ -2006,11 +2029,13 @@ menu_tree_strip_duplicate_children (MenuTree       *tree,
             {
               /* nuke it! */
               menu_layout_node_unlink (n);
+	      simple_nodes = g_slist_delete_link (simple_nodes, tmp);
+	      tmp = prev;
             }
         }
 
       prev = tmp;
-      tmp = tmp->next;
+      tmp = next;
     }
 
   g_slist_free (simple_nodes);
@@ -2021,12 +2046,14 @@ menu_tree_strip_duplicate_children (MenuTree       *tree,
    * the list is backward.
    */
   menu_layout_nodes = g_slist_sort (menu_layout_nodes,
-                             node_menu_compare_func);
+				    node_menu_compare_func);
 
   prev = NULL;
   tmp = menu_layout_nodes;
   while (tmp != NULL)
     {
+      GSList *next = tmp->next;
+
       if (prev)
         {
           MenuLayoutNode *p = prev->data;
@@ -2039,11 +2066,13 @@ menu_tree_strip_duplicate_children (MenuTree       *tree,
                */
               move_children (n, p);
               menu_layout_node_unlink (n);
+	      menu_layout_nodes = g_slist_delete_link (menu_layout_nodes, tmp);
+	      tmp = prev;
             }
         }
 
       prev = tmp;
-      tmp = tmp->next;
+      tmp = next;
     }
 
   g_slist_free (menu_layout_nodes);
@@ -2063,6 +2092,8 @@ menu_tree_strip_duplicate_children (MenuTree       *tree,
       tmp = move_nodes;
       while (tmp != NULL)
         {
+	  GSList *next = tmp->next;
+
           if (prev)
             {
               MenuLayoutNode *p = prev->data;
@@ -2077,11 +2108,13 @@ menu_tree_strip_duplicate_children (MenuTree       *tree,
                                 menu_layout_node_move_get_old (p),
                                 menu_layout_node_move_get_new (p));
                   menu_layout_node_unlink (n);
+		  move_nodes = g_slist_delete_link (move_nodes, tmp);
+		  tmp = prev;
                 }
             }
 
           prev = tmp;
-          tmp = tmp->next;
+          tmp = next;
         }
 
       g_slist_free (move_nodes);
@@ -2327,9 +2360,39 @@ menu_tree_force_reload (MenuTree *tree)
   tree->layout = NULL;
 }
 
+typedef struct
+{
+  DesktopEntrySet *set;
+  const char      *category;
+} GetByCategoryForeachData;
+
+static void
+get_by_category_foreach (const char               *file_id,
+			 DesktopEntry             *entry,
+			 GetByCategoryForeachData *data)
+{
+  if (desktop_entry_has_category (entry, data->category))
+    desktop_entry_set_add_entry (data->set, entry, file_id);
+}
+
+static void
+get_by_category (DesktopEntrySet *entry_pool,
+		 DesktopEntrySet *set,
+		 const char      *category)
+{
+  GetByCategoryForeachData data;
+
+  data.set      = set;
+  data.category = category;
+
+  desktop_entry_set_foreach (entry_pool,
+			     (DesktopEntrySetForeachFunc) get_by_category_foreach,
+			     &data);
+}
+
 static DesktopEntrySet *
-process_include_rules (MenuLayoutNode     *layout,
-                       EntryDirectoryList *list)
+process_include_rules (MenuLayoutNode  *layout,
+		       DesktopEntrySet *entry_pool)
 {
   DesktopEntrySet *set = NULL;
 
@@ -2346,7 +2409,7 @@ process_include_rules (MenuLayoutNode     *layout,
           {
             DesktopEntrySet *child_set;
 
-            child_set = process_include_rules (child, list);
+            child_set = process_include_rules (child, entry_pool);
 
             if (set == NULL)
               {
@@ -2381,7 +2444,7 @@ process_include_rules (MenuLayoutNode     *layout,
           {
             DesktopEntrySet *child_set;
 
-            child_set = process_include_rules (child, list);
+            child_set = process_include_rules (child, entry_pool);
 
             if (set == NULL)
               {
@@ -2411,7 +2474,7 @@ process_include_rules (MenuLayoutNode     *layout,
           {
             DesktopEntrySet *child_set;
 
-            child_set = process_include_rules (child, list);
+            child_set = process_include_rules (child, entry_pool);
 
             if (set == NULL)
               {
@@ -2428,8 +2491,14 @@ process_include_rules (MenuLayoutNode     *layout,
 
         if (set != NULL)
           {
-            /* Now invert the result */
-            entry_directory_list_invert_set (list, set);
+	    DesktopEntrySet *inverted;
+
+	    /* Now invert the result */
+	    inverted = desktop_entry_set_new ();
+	    desktop_entry_set_union (inverted, entry_pool);
+	    desktop_entry_set_subtract (inverted, set);
+	    desktop_entry_set_unref (set);
+	    set = inverted;
           }
 	menu_verbose ("Processed <Not>\n");
       }
@@ -2438,7 +2507,7 @@ process_include_rules (MenuLayoutNode     *layout,
     case MENU_LAYOUT_NODE_ALL:
       menu_verbose ("Processing <All>\n");
       set = desktop_entry_set_new ();
-      entry_directory_list_get_all_desktops (list, set);
+      desktop_entry_set_union (set, entry_pool);
       menu_verbose ("Processed <All>\n");
       break;
 
@@ -2449,15 +2518,14 @@ process_include_rules (MenuLayoutNode     *layout,
 	menu_verbose ("Processing <Filename>%s</Filename>\n",
 		      menu_layout_node_get_content (layout));
 
-        entry = entry_directory_list_get_desktop (list,
-                                                  menu_layout_node_get_content (layout));
+        entry = desktop_entry_set_lookup (entry_pool,
+					  menu_layout_node_get_content (layout));
         if (entry != NULL)
           {
             set = desktop_entry_set_new ();
             desktop_entry_set_add_entry (set,
                                          entry,
                                          menu_layout_node_get_content (layout));
-            desktop_entry_unref (entry);
           }
 	menu_verbose ("Processed <Filename>%s</Filename>\n",
 		      menu_layout_node_get_content (layout));
@@ -2468,9 +2536,7 @@ process_include_rules (MenuLayoutNode     *layout,
       menu_verbose ("Processing <Category>%s</Category>\n",
 		    menu_layout_node_get_content (layout));
       set = desktop_entry_set_new ();
-      entry_directory_list_get_by_category (list,
-                                            menu_layout_node_get_content (layout),
-                                            set);
+      get_by_category (entry_pool, set, menu_layout_node_get_content (layout));
       menu_verbose ("Processed <Category>%s</Category>\n",
 		    menu_layout_node_get_content (layout));
       break;
@@ -2506,10 +2572,9 @@ process_layout (MenuTree          *tree,
                 MenuLayoutNode    *layout,
                 DesktopEntrySet   *allocated)
 {
-  EntryDirectoryList *app_dirs;
-  EntryDirectoryList *dir_dirs;
   MenuLayoutNode     *layout_iter;
   MenuTreeDirectory  *directory;
+  DesktopEntrySet    *entry_pool;
   DesktopEntrySet    *entries;
   gboolean            deleted;
   DesktopEntrySet    *allocated_set;
@@ -2528,12 +2593,13 @@ process_layout (MenuTree          *tree,
   deleted = FALSE;
   only_unallocated = FALSE;
 
-  app_dirs = menu_layout_node_menu_get_app_dirs (layout);
-  dir_dirs = menu_layout_node_menu_get_directory_dirs (layout);
-
   entries = desktop_entry_set_new ();
   allocated_set = desktop_entry_set_new ();
 
+  entry_pool = desktop_entry_set_new ();
+  entry_directory_list_get_all_desktops (menu_layout_node_menu_get_app_dirs (layout),
+					 entry_pool);
+
   layout_iter = menu_layout_node_get_children (layout);
   while (layout_iter != NULL)
     {
@@ -2574,7 +2640,7 @@ process_layout (MenuTree          *tree,
               {
                 DesktopEntrySet *rule_set;
 
-                rule_set = process_include_rules (rule, app_dirs);
+                rule_set = process_include_rules (rule, entry_pool);
                 if (rule_set != NULL)
                   {
                     desktop_entry_set_union (entries, rule_set);
@@ -2606,7 +2672,7 @@ process_layout (MenuTree          *tree,
               {
                 DesktopEntrySet *rule_set;
 
-                rule_set = process_include_rules (rule, app_dirs);
+                rule_set = process_include_rules (rule, entry_pool);
                 if (rule_set != NULL)
                   {
                     desktop_entry_set_subtract (entries, rule_set);
@@ -2631,7 +2697,7 @@ process_layout (MenuTree          *tree,
 	    /*
              * The last <Directory> to exist wins, so we always try overwriting
              */
-            entry = entry_directory_list_get_directory (dir_dirs,
+            entry = entry_directory_list_get_directory (menu_layout_node_menu_get_directory_dirs (layout),
                                                         menu_layout_node_get_content (layout_iter));
 
             if (entry != NULL)
@@ -2680,6 +2746,8 @@ process_layout (MenuTree          *tree,
       layout_iter = menu_layout_node_get_next (layout_iter);
     }
 
+  desktop_entry_set_unref (entry_pool);
+
   directory->only_unallocated = only_unallocated;
 
   if (!directory->only_unallocated)
@@ -2711,7 +2779,6 @@ process_layout (MenuTree          *tree,
       return NULL;
     }
 
-  directory->entries = NULL;
   desktop_entry_set_foreach (entries,
                              (DesktopEntrySetForeachFunc) entries_listify_foreach,
                              directory);
@@ -2756,7 +2823,7 @@ process_layout (MenuTree          *tree,
         {
           directory->entries = g_slist_delete_link (directory->entries,
                                                    tmp);
-          menu_tree_entry_unref (entry);
+          menu_tree_entry_unref_and_unset_parent (entry);
         }
 
       tmp = next;
@@ -2789,7 +2856,7 @@ process_only_unallocated (MenuTreeDirectory *directory,
             {
               directory->entries = g_slist_delete_link (directory->entries,
                                                         tmp);
-              menu_tree_entry_unref (entry);
+              menu_tree_entry_unref_and_unset_parent (entry);
             }
 
           tmp = next;
@@ -2811,7 +2878,7 @@ process_only_unallocated (MenuTreeDirectory *directory,
         {
           directory->subdirs = g_slist_delete_link (directory->subdirs,
                                                     tmp);
-          menu_tree_directory_unref (subdir);
+          menu_tree_directory_unref_and_unset_parent (subdir);
         }
 
       tmp = next;
